import base64
import json
import os
import pendulum
import pyotp
import re
import typer
import yaml

from app.logs import Log
from app.settings import Settings
from app.web import Web
from pathlib import Path

home = str(Path.home())
settings = Settings.config()
if "loglevel" in settings["vikunja-cli"]["general"]:
    log = Log("auth", settings["vikunja-cli"]["general"]["loglevel"])
else:
    log = Log("auth")


class Auth:
    def __init__(self):
        if not os.path.isfile(f"{home}/.config/vikunja-cli/jwt.yaml"):
            log.warning(f"Vikunja-cli can't get JWT")
            log.info(f"Launch first run")
            self._create_jwt_file()

    def get_jwt(self):
        try:
            with open(f"{home}/.config/vikunja-cli/jwt.yaml", "r") as jwt_file:
                jwt = yaml.safe_load(jwt_file)
        except (OSError, ValueError, BaseException, yaml.YAMLError):
            log.critical(f"Failed to JWT token !")
            raise typer.Exit(code=1)

        jwttoken = jwt["jwttoken"]
        if not jwttoken:
            log.debug("JWT token is not defined, need auth")
            jwttoken = self.auth()
        if self._is_expired(jwttoken):
            log.debug("JWT token is expired, need auth")
            jwttoken = self.auth()
        elif self._need_renew(token=jwttoken, ttl=60 * 60 * 24):
            log.debug(
                f"JWT token is close to expire (in {60*60*24} secondes). Need reenw"
            )
            jwttoken = self.renew(jwttoken)
        return jwttoken

    def auth(self):
        totp = pyotp.TOTP(settings["vikunja-cli"]["auth"]["totp_passcode"])
        payload = {
            "long_token": True,
            "password": settings["vikunja-cli"]["auth"]["password"],
            "totp_passcode": totp.now(),
            "username": settings["vikunja-cli"]["auth"]["username"],
        }
        r = Web.request(method="POST", path="/login", payload=json.dumps(payload))
        token = r.json()["token"]
        self._create_jwt_file(token)
        return token

    def renew(self, token):
        totp = pyotp.TOTP(settings["vikunja-cli"]["auth"]["totp_passcode"])
        headers = {"Authorization": f"Bearer {token}"}
        r = Web.request(token=token, method="POST", path="/login", payload=payload)

        token = r.json()["token"]
        self._create_jwt_file(token)
        return token

    def _create_jwt_file(self, token: str = ""):
        try:
            Path(f"{home}/.config/vikunja-cli/").mkdir(parents=True, exist_ok=True)
        except OSError:
            log.critical(f"can't create {home}/.config/vikunja-cli !")
            raise typer.Exit(code=1)
        jwt = {"jwttoken": token}
        try:
            with open(f"{home}/.config/vikunja-cli/jwt.yaml", "w") as config_file:
                yaml.dump(jwt, config_file, default_flow_style=False)
        except (OSError, ValueError, BaseException, yaml.YAMLError):
            log.critical(
                f"Can't write config file {home}/.config/vikunja-cli/jwt.yaml !"
            )
            raise typer.Exit(code=1)

        log.info(f"JWT token is correctly saved")

    def _is_expired(self, token):
        token_expiration = self._get_token_expiration(token)
        log.debug(f"Expire timestamp is {token_expiration}")
        expiration_date = pendulum.from_timestamp(token_expiration)
        log.debug(f"Expiration is {expiration_date}")
        if expiration_date.is_past():
            log.warning(f"JWT is expired")
            return True
        else:
            return False

    def _need_renew(self, token: str, ttl: int = 3600):
        token_expiration = self._get_token_expiration(token)
        expiration_date = pendulum.from_timestamp(token_expiration)
        log.debug(
            f"Date required to renew the token {expiration_date.subtract(seconds=ttl)}"
        )
        if expiration_date.subtract(seconds=ttl).is_past():
            log.info("JWT token will be renewed")
            return True
        else:
            return False

    def _get_token_expiration(self, token):
        tokenSplit = token.split(".")
        token_payload = json.loads(base64.b64decode(tokenSplit[1] + "=="))
        token_expiration = token_payload["exp"]
        return token_expiration
