import json
import pendulum
import re
import typer

import pprint

from app.auth import Auth
from app.logs import Log
from app.settings import Settings
from app.web import Web
from dateparser.search import search_dates
from tzlocal import get_localzone_name
from rich.console import Console
from rich.table import Table
from rich import print


settings = Settings.config()
if "loglevel" in settings["vikunja-cli"]["general"]:
    log = Log("Namespaces", settings["vikunja-cli"]["general"]["loglevel"])
else:
    log = Log("Namespaces")

auth = Auth()
app = typer.Typer()
console = Console()
jwt = auth.get_jwt()

lists_name = None

@app.command()
def list():
    r = Web.request(
        token=jwt,
        method="GET",
        path="/tasks/all",
        payload={"sort_by": "priority", "order_by": "desc"},
    )
    tasks = r.json()

    table = Table("ID", "Title", "Description", "List", "Owner", "Created", "Due date")
    done_list = []
    for task in tasks:
        if task["done"]:
            done_list.append(task)
        else:
            table = _generate_task_table(task, table)
    for task in done_list:
        table = _generate_task_table(task, table)

    console.print(table)


@app.command()
def add(
    title: str,
    list: str = typer.Option(None, help="List where you want to add this task"),
    list_id: int = typer.Option(None, help="List ID where you want to add this task"),
    no_quick: bool = typer.Option(False, help="Disable quick add on this task"),
):
    payload = {"title": title}
    quick_list = _get_quick_list(title)
    if quick_list and no_quick == False:
        log.debug(f'"{quick_list}" found')
        list_id = _get_list_id_by_name(quick_list.replace("+", ""))
        payload["title"] = title.replace(quick_list, "")
    if (
        list is None
        and list_id is None
        and not "default_list" in settings["vikunja-cli"]["general"]
    ):
        print(
            "[bold red]You need to specify a list name (--list) or a list id (--list-id)[/]"
        )
        print(
            "[bold red]You can also set a default value with [/]`vikunja list set-default`"
        )
        raise typer.Exit(code=1)
    elif list and list_id:
        print("[bold red]You cannot specify list and list-id at the same time[/]")
        raise typer.Exit(code=1)
    elif list:
        log.debug(f"List is set with arguments {list}")
        list_id = _get_list_id_by_name(list)
    elif list_id:
        log.debug(f"List_id is set with arguments {list_id}")
        pass
    elif "default_list" in settings["vikunja-cli"]["general"]:
        log.debug(
            f"default_list is set to {settings['vikunja-cli']['general']['default_list']}"
        )
        if type(settings["vikunja-cli"]["general"]["default_list"]) == int:
            list_id = settings["vikunja-cli"]["general"]["default_list"]
        else:
            print("[bold red]default_list need to be an number.[/]")
            print(
                "[bold red]You can reset this value with [/]`vikunja list set-default`"
            )
            raise typer.Exit(code=1)
    else:
        print("Hu ? :thinking_face:")
        raise typer.Exit(code=1)

    log.info("find date")
    log.debug(title)
    if no_quick == False:
        date = search_dates(title)
        if date is None:
            log.debug("No date found")
        else:
            if len(date) > 1:
                print("[bold red] You can specify only one date in task[/]")
                raise typer.Exit(code=1)
            else:
                payload.update(
                    {
                        "due_date": pendulum.instance(date[0][1])
                        .set(tz=get_localzone_name())
                        .to_rfc3339_string()
                    }
                )

    if no_quick == False:
        priority = _get_quick_priority(title)
        if priority:
            payload["title"] = title.replace(priority, "")
            payload["title"] = payload["title"].replace("\\", "")
            payload.update({"priority": int(priority.replace("!", ""))})
        log.debug(f"Priority is {priority}")

    log.debug(f"list_id is : {list_id}")
    r = Web.request(
        token=jwt,
        method="PUT",
        path=f"/lists/{list_id}",
        payload=json.dumps(payload),
    )
    task = r.json()
    print("[bold green]Task succcessfully created[/]")
    table = Table("ID", "Title", "Description", "List", "Owner", "Created", "Due date")
    table = _generate_task_table(task, table)
    console.print(table)

@app.command()
def done(id:int):
    payload={
        "done": True
    }
    r = Web.request(
        token=jwt,
        method="POST",
        path=f"/tasks/{id}",
        payload=json.dumps(payload),
    )
    print(f"[bold green]Task {id} completed correctly[/]")
    r = Web.request(
        token=jwt,
        method="GET",
        path=f"/tasks/{id}",
        payload={},
    )
    task = r.json()
    table = Table("ID", "Title", "Description", "List", "Owner", "Created", "Due date")
    table = _generate_task_table(task, table)
    console.print(table)

def _get_quick_list(title):
    regex = "\+[a-zA-Z]+"
    try:
        found = re.search(regex, title)
        return found.group(0)
    except AttributeError:
        return False


def _get_quick_priority(title):
    regex = "\![0,1,2,3,4,5]{1}"
    try:
        found = re.search(regex, title)
        return found.group(0)
    except AttributeError:
        return False


def _get_list_id_by_name(name: str):
    r = Web.request(token=jwt, method="GET", path="/lists", payload={"s": name})
    list_req = r.json()
    if not list_req:
        print(f'[bold red]List "{list}" doesn\'t exist[/]')
        raise typer.Exit(code=1)
    else:
        if len(list_req) > 1:
            print(
                f'[bold red]Many list with the name "{list}". Specify id with --list-id[/]'
            )
            raise typer.Exit(code=1)
        else:
            list_id = list_req[0]["id"]
    return list_id


def _generate_task_table(task: dict, table: Table):
    global lists_name
    priority = [
        "  ",
        ":ZZZ: ",
        ":warning:  ",
        ":small_red_triangle: ",
        ":exclamation: ",
        ":fire: ",
    ]
    if lists_name is None:
        lists_name = _get_lists_name()
    created = pendulum.parse(task["created"])
    created = created.format("dddd Do MMMM YYYY HH:mm:ss")
    due_date = pendulum.parse(task["due_date"])
    title_append = ""
    if task["done"]:
        title_append = ":white_check_mark: "
    else:
        title_append = priority[task["priority"]]

    if due_date.timestamp() < 1:
        due_date = "No due date"
    elif due_date.is_past():
        due_date = f"[red3]{due_date.diff_for_humans()} ({due_date.format('dddd Do MMMM YYYY HH:mm:ss')})[/]"
        due_date = f"[red3]{due_date.format('dddd Do MMMM YYYY HH:mm:ss')}[/]"
    else:
        due_date = due_date.format("dddd Do MMMM YYYY HH:mm:ss")
    table.add_row(
        f"{task['id']}",
        f"{title_append}{task['title']}",
        f"{task['description']}",
        f"[#{lists_name[task['list_id']]['hex_color']}]{lists_name[task['list_id']]['name']}[/]",
        f"{task['created_by']['username']}",
        f"{created}",
        f"{due_date}",
    )

    return table


def _get_lists_name():
    r = Web.request(token=jwt, method="GET", path="/lists")
    lists = r.json()
    lists_dict = {}
    for list in lists:
        lists_dict.update(
            {list["id"]: {"name": list["title"], "hex_color": list["hex_color"]}}
        )
    return lists_dict


if __name__ == "__main__":
    app()
