import json
import requests
import typer

from app.logs import Log
from app.settings import Settings


settings = Settings.config()
if "loglevel" in settings["vikunja-cli"]["general"]:
    log = Log("Web", settings["vikunja-cli"]["general"]["loglevel"])
else:
    log = Log("Web")


class Web:
    """Web."""

    @staticmethod
    def request(
        token: str = None,
        method: str = "GET",
        url: str = None,
        path: str = "/info",
        payload: dict = {},
    ):
        """request.

        :param method: POST, PUT, DELETE, GET, PATCH
        :type method: str
        :param path: The URI
        :type path: str
        :param payload: payload to send it in request (ex in POST)
        :type payload: str
        """
        method_allowed = ["GET", "POST", "PUT", "PATCH", "DELETE"]
        if method not in method_allowed:
            log.critical(f"Method ({method}) is not allowed in {method_allowed}")
            raise typer.Exit(code=1)

        headers = {
            "Content-Type": "application/json",
            "Accept": "application/json",
        }
        if url:
            url = url
        else:
            url = f"{settings['vikunja-cli']['general']['url']}{path}"
        if token:
            headers.update({"Authorization": f"Bearer {token}"})

        data = {}
        params = {}
        if method == "GET":
            params = payload
        else:
            data = payload

        log.debug(f"Using URL : {url}")
        log.debug(f"Using payload : {payload}")
        log.debug(f"Using header : {headers}")
        try:
            req = requests.request(
                f"{method}",
                url,
                headers=headers,
                params=params,
                data=data,
            )
            req.raise_for_status()
        except (
            requests.exceptions.Timeout,
            requests.exceptions.ReadTimeout,
            requests.exceptions.ConnectTimeout,
        ) as err:
            log.critical(f"Connection to {url} timeout")
            raise typer.Exit(code=1)
        except requests.exceptions.HTTPError as err:
            if Web.is_json(err.response.text):
                response = json.loads(err.response.text)
                if "message" in response.keys():
                    message = response["message"]
                else:
                    message = err.response.text
            else:
                message = "not message"
            log.critical(f"An error occured. {message} ({err.response.status_code})")
            raise typer.Exit(code=1)
        except requests.exceptions.URLRequired as err:
            log.critical(f"An error occured. URL is malformated ({url})")
            raise typer.Exit(code=1)

        return req

    @staticmethod
    def is_json(myjson):
        try:
            json.loads(myjson)
        except ValueError as e:
            return False
        return True
